/******************************************************************************
 * The copyright in this software is being made available under the BSD        *
 * License, included below. This software may be subject to other third party  *
 * and contributor rights, including patent rights, and no such rights are     *
 * granted under this license.                                                 *
 *    <OWNER> = ISO                                                            *
 *    <ORGANIZATION> = ISO                                                     *
 *    <YEAR> = 2015                                                            *
 *    Copyright (c) 2012-2015, ISO                                             *
 *                                                                             *
 * All rights reserved.                                                        *
 * Redistribution and use in source and binary forms, with or without          *
 * modification, are permitted provided that the following conditions are met: *
 *       * Redistributions of source code must retain the above copyright      *
 *         notice,this list of conditions and the following disclaimer.        *
 *       * Redistributions in binary form must reproduce the above             *
 *         copyright notice, this list of conditions and the following         *
 *         disclaimer in the documentation and/or other materials provided     *
 *         with the distribution.                                              *
 *       * Neither the name of the <ORGANIZATION> nor the names of its         *
 *         contributors may be used to endorse or promote products derived     *
 *         from this software without specific prior written permission.       *
 *                                                                             *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" *
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE  *
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE   *
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR         *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF        *
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS    *
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN     *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)     *
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE  *
 * POSSIBILITY OF SUCH DAMAGE.                                                 *
 ******************************************************************************/

/** @file
 *  @brief Header file containing declarations of shared data types for the JPEG-HDR mobile encoder, decoder, and tonemapper
 *  libraries.
 *
 *  This header file contains declarations of data structures, enumerations, type definitions, and constants that are shared by
 *  the JPEG-HDR mobile encoder, decoder, and tonemapper libraries.
 */

/** @defgroup share Shared Data Types
 *  @brief Declares data types shared by the JPEG-HDR mobile encoder, decoder, and tonemapper libraries.
 *  @{
 */

#ifndef _JH_DATATYPES_H
#define _JH_DATATYPES_H
#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @brief Defines the pixel type of an image.
 */
typedef enum jh_pixel_type_ts {
    JH_RGB8         = 1,            /**< Interleaved (packed) RGB 8-bit data. */
    JH_RGBF         = 2,            /**< Interleaved (packed) RGB 32-bit floating point data. */
    JH_YUV444I8     = 3,            /**< Interleaved (packed) YUV 4:4:4 8-bit data. */
    JH_GRAY8        = 4,            /**< Monochrome 8-bit data. */
    JH_GRAYF        = 5             /**< Monochrome 32-bit floating point data. */
} jh_pixel_type_t;

/** @brief Defines result codes, which are provided for debugging purposes.
 */
typedef enum jh_result_ts {
    JH_OK                   =  0,   /**< The operation completed successfully. */
    JH_MEMORY               =  1,   /**< Memory allocation failed. */
    JH_CANCELED             =  2,   /**< The operation was canceled. */
    JH_INVALID_ARGUMENT     =  3,   /**< One or more input arguments are not valid. */
    JH_JPEG_ERROR           =  4,   /**< An error occurred in the JPEG library. */
    JH_JPEG_FORMAT_ERROR    =  5,   /**< The file is not in JPEG format or is corrupted. */
    JH_UNSUPPORTED          =  6,   /**< The requested functionality is not supported. */
    JH_NOT_IMPLEMENTED      =  7,   /**< The requested functionality has not yet been implemented. */
    JH_USER_STREAM_ERROR    =  8,   /**< An error occurred in a user-provided stream. */
    JH_M11_FORMAT_ERROR     =  9,   /**< An error occurred during parsing of the APP11 data. */
    JH_M11_VERSION_ERROR    = 10,   /**< The version number of the DD HDR APP11 is not valid. The current version is 2.0. */
    JH_ERROR                = 11,   /**< An unclassified error occurred. */
    JH_ENCODER_ERROR        = 12,   /**< An encoder error occurred. */
    JH_HEADER_NO_CHECKSUM   = 13,   /**< No checksum field in header (warning). */
    JH_INVALID_CHECKSUM     = 14,   /**< Computed checksum does not match value in header. */
    JH_CHECKSUM_PARSE_ERROR = 15,   /**< An error occurred while computing the checksum for the stream. */
    JH_CHECKSUM_WRITE_ERROR = 16    /**< An error occurred while writing the checksum to the stream. */
} jh_result_t;

/** @brief Maximum size of the string in the jh_error_t @c description field.
 */
#define JH_MSG_SIZE 200

/** \struct jh_error_t
 */
/** @brief Defines the error reporting structure.
 *
 *   The structure should be allocated by the user.
 */
typedef struct jh_error_ts {
    jh_result_t code;               /**< Result code. */
    char description[JH_MSG_SIZE];  /**< User-provided string containing the error description. Maximum length is #JH_MSG_SIZE
                                         characters. */
} jh_error_t;

/** @brief Defines the structure that holds the image data.
 */
typedef struct jh_image_ts {
    int columns;                    /**< Number of columns in the image. */
    int rows;                       /**< Number of rows in the image. */
    int stride;                     /**< Distance between scanlines, in bytes. */
    jh_pixel_type_t pixel_type;     /**< Pixel type. */
    void *data;                     /**< Image memory consisting of stride-size scanlines containing packed pixels. */
    int lut_size;                   /**< Size of the decoding LUT. The size of the LUT is 256 for 8-bit data.*/
    float *decoding_lut;            /**< Decoding LUT to convert gamma-corrected data into linear space. */
    void *loglum;                   /**< For internal use. */
} jh_image_t;

/** \struct jh_monitor_t
 */
/** @brief Defines the means to control execution of JPEG-HDR mobile library functions.
 *
 *  If a caller of a JPEG-HDR function decides to interrupt execution, the caller must set the value of
 *  the @c cancel field to 1. All JPEG-HDR functions that accept jh_monitor_t type objects monitor the value
 *  of @c cancel and abort their execution as soon as the value of @c cancel becomes non-zero.
 *  The functions returns 0 to indicate failure, and an error code in the jh_error_t object is set to @c JH_CANCELED.
 *
 *  The caller must set the value of @c cancel to 0 prior to passing the jh_monitor_t object to any function.
 */
typedef struct jh_monitor_ts {
    volatile int cancel;        /**< Cancel flag. If set, the execution of a library function is aborted. */
    volatile float progress;    /**< NOT YET SUPPORTED. */
} jh_monitor_t;

/** @brief  Defines a container for the JPEG APP or COM segment.
 */
typedef struct jh_marker_segment_ts {

    /** @brief JPEG marker identifier, for example, APP6 or COM.
     */
    unsigned char marker;

    /** @brief Size of the marker segment data, in bytes.
     *
     *  This field does not include the 2 bytes for the @c length field of a marker segment. Therefore, the size of this field is
     *  2 bytes smaller than the @c length field, as stored in the marker in the file.
     */
    unsigned int length;

    /** @brief Marker segment data.
     */
    unsigned char *data;
} jh_marker_segment_t;

/** \struct jh_marker_list_t
 */
/** @brief Defines the user-implemented interface to a container for JPEG COM and APP segment data.
 */
typedef struct jh_marker_list_ts {

    /** @brief User-implemented function that adds a segment to the list by copying.
     *  @param [in] markers   Object of type jh_marker_list_t.
     *  @param [in] marker_id JPEG marker ID per the JPEG specification.
     *  @param [in] size1     Size of the first data buffer (<code>data1</code>), in bytes.
     *  @param [in] data1     First data buffer.
     *  @param [in] size2     Size of the second data buffer (<code>data2</code>), in bytes. The value of this parameter can
                              be 0.
     *  @param [in] data2     Second data buffer. Can be NULL.
     *  @return 0 if the function does not succeed.
     */
    int (*add)(struct jh_marker_list_ts* markers,
               const unsigned char marker_id,
               const unsigned int size1, 
               const unsigned char* data1,
               const unsigned int size2, 
               const unsigned char* data2
               );

    /** @brief User-implemented function that deletes the specified segment from a marker container and sets the segment to
     *  NULL if the function succeeds.
     *  @param [in]     markers Object of type jh_marker_list_t.
     *  @param [in,out] segment Segment in the markers object that needs to be deleted.
     *  @return 0 if the function does not succeed.
     */
    int (*jh_delete_segment)(struct jh_marker_list_ts* markers,
                          jh_marker_segment_t** segment
                         );

    /** @brief User-implemented function that moves the current segment to the next segment and returns the next segment to the
     *  caller.
     *  @param [in]  markers Object of type jh_marker_list_t.
     *  @param [out] segment Next segment, if one exists, otherwise NULL.
     *  @return 0 if the function does not succeed.
     */
    int (*jh_get_next)(struct jh_marker_list_ts* markers,
                    jh_marker_segment_t** segment
                    );
        
    /** @brief User-implemented function that moves the current segment to the previous segment and returns the previous
     *  segment to the caller.
     *  @param [in]  markers Object of type jh_marker_list_t.
     *  @param [out] segment Previous segment, if one exists, otherwise NULL.
     *  @return 0 if the function does not succeed.
     */
    int (*jh_get_prev)(struct jh_marker_list_ts* markers,
                    jh_marker_segment_t** segment
                    );

    /** @brief User-implemented function that gets the current segment.
     *  @param [in]  markers Object of type jh_marker_list_t
     *  @param [out] segment Current segment, if one exists, otherwise NULL.
     *  @return 0 if the function does not succeed.
     */
    int (*get)(struct jh_marker_list_ts* markers,
               jh_marker_segment_t** segment
               );

    /** @brief User-implemented function that empties the container and releases all resources acquired by the list.
     *  @param [in] markers Object of type jh_marker_list_t.
     */
     void (*empty)(struct jh_marker_list_ts* markers);
} jh_marker_list_t;

/** @brief Value that defines the beginning of the stream.
 */
#define JH_SEEK_SET 0

/** @brief Value that defines the current position.
 */
#define JH_SEEK_CUR 1

/** @brief Value that defines the end of the stream.
 */
#define JH_SEEK_END 2

/** @brief Forward declaration of jh_stream_t.
 */
struct jh_stream_ts;

/** @brief Defines the buffer manager interface, which allows stream manipulation for memory-buffer based JPEG implementations.
 */
typedef struct {

    /** @brief Initializes the memory buffer manager. This function is called before the first use of the buffer.
     *  @param [in,out] stream    Stream that is being read or written.
     *  @param [out]    buf       Stream buffer.
     *  @param [out]    buf_size  Size of the stream buffer.
     *  @return Non-zero if the function succeeds, 0 if the function does not succeed.
     */
    int (*on_init)(struct jh_stream_ts *stream,
                   unsigned char **buf,
                   size_t *buf_size
                   );

    /** @brief Reinitializes the memory buffer. This function is called when a reader or a writer reaches the end of the buffer.
     *  @param [in,out] stream    Stream that is being read or written.
     *  @param [out]    buf       Stream buffer.
     *  @param [out]    buf_size  Size of the stream buffer.
     *  @return Non-zero if the function succeeds, 0 if the function does not succeed.
     */
    int (*on_empty)(struct jh_stream_ts *stream,
                    unsigned char **buf,
                    size_t *buf_size
                    );

    /** @brief Finalizes the stream. This function is called when a reader or writer completes reading or writing the stream.
     *  @param [in,out] stream          Stream that is being read or written.
     *  @param [in]     free_in_buffer  Unread or unwritten bytes left in the buffer.
     *  @return Non-zero if the function succeeds, 0 if the function does not succeed.
     */
    int (*on_term)(struct jh_stream_ts *stream,
                   size_t free_in_buffer
                  );
} jh_buf_mgr_t;

/** \struct jh_stream_t
 */
/** @brief Defines the interface to the user-implemented stream.
 */
typedef struct jh_stream_ts {

    /** @brief User-implemented function that reads the number of bytes in a stream to a buffer.
     *  @param [in] stream  Data stream.
     *  @param [in] buffer  Address where the data is to be stored.
     *  @param [in] bytes   Number of bytes to be read.
     *  @return Number of bytes read.
     */
    int (*read)(struct jh_stream_ts *stream,
                unsigned char *buffer,
                size_t bytes
                );

    /** @brief User-implemented function that writes the number of bytes in a buffer to a stream.
     *  @param [in] stream  Output data stream.
     *  @param [in] buffer  Address where the data is stored.
     *  @param [in] bytes   Number of bytes to be written.
     *  @return Number of bytes written.
     */
    int (*write)(struct jh_stream_ts *stream,
                 unsigned char *buffer,
                 size_t bytes
                 );

    /** @brief User-implemented function that navigates through a file and sets the reading position.
     *  @param [in] stream    Output data stream.
     *  @param [in] position  Byte offset relative to the origin.
     *  @param [in] origin    Starting point for the seek. Possible values are #JH_SEEK_SET, #JH_SEEK_CUR, and #JH_SEEK_END.
     *  @return 0 if the function does not succeed.
     */
    int (*seek)(struct jh_stream_ts *stream,
                long long position,
                unsigned origin
                );

    /** @param pri Buffer manager. The manager optimizes support for buffered input/output for IJG JPEG implementation.
     */
    jh_buf_mgr_t *pri;

} jh_stream_t;


/**@}*/  /* end of defgroup share */

#ifdef __cplusplus
}
#endif

#endif /*_JH_DATATYPES_H*/
