/******************************************************************************
 * The copyright in this software is being made available under the BSD        *
 * License, included below. This software may be subject to other third party  *
 * and contributor rights, including patent rights, and no such rights are     *
 * granted under this license.                                                 *
 *    <OWNER> = ISO                                                            *
 *    <ORGANIZATION> = ISO                                                     *
 *    <YEAR> = 2015                                                            *
 *    Copyright (c) 2012-2015, ISO                                             *
 *                                                                             *
 * All rights reserved.                                                        *
 * Redistribution and use in source and binary forms, with or without          *
 * modification, are permitted provided that the following conditions are met: *
 *       * Redistributions of source code must retain the above copyright      *
 *         notice,this list of conditions and the following disclaimer.        *
 *       * Redistributions in binary form must reproduce the above             *
 *         copyright notice, this list of conditions and the following         *
 *         disclaimer in the documentation and/or other materials provided     *
 *         with the distribution.                                              *
 *       * Neither the name of the <ORGANIZATION> nor the names of its         *
 *         contributors may be used to endorse or promote products derived     *
 *         from this software without specific prior written permission.       *
 *                                                                             *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" *
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE  *
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE   *
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR         *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF        *
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS    *
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN     *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)     *
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE  *
 * POSSIBILITY OF SUCH DAMAGE.                                                 *
 ******************************************************************************/

/** @file
 *  @brief Header file containing declarations of functions and data structures for the JPEG-HDR mobile decoder library.
 *
 *  This file contains declarations of the data structures and functions that are used by the JPEG-HDR mobile decoder library.
 */

#ifndef _JH_DECODER_H
#define _JH_DECODER_H

#include "jh_datatypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/** @defgroup decodint Decoder Interface
 *  @brief Defines the API for the JPEG-HDR mobile decoder library.
 *  @{
 */

/** \struct jh_decoder_params_t
 */
 /** @brief Defines the decoder parameters.
 */
typedef struct jh_decoder_params_ts {
    int num;                                    /**< Scaling numerator. The value depends on the JPEG library used. */
    int denom;                                  /**< Scaling denominator. The value depends on the JPEG library used.*/
    jh_marker_list_t *marker_list;              /**< User-defined container for storing JPEG APP and COM data. */
} jh_decoder_params_t;

/** @brief Decodes the JPEG-HDR data and returns the pointer to an HDR image.
 *  
 *  All the memory associated with the HDRI handle is allocated in the jh_decode() @c buffer parameter. See
 *  jh_dispose_image().
 *
 *  @attention jh_decode() should not be called if jh_is_hdr() returns 0.
 *
 *  @par
 *  jh_dispose_image() must be called after jh_decode() is called and as many times as jh_decode() is successfully executed.
 *  @return Pointer to the HDR image if the function succeeds. NULL if the function does not succeed.
*/
jh_image_t* jh_decode(
    jh_stream_t*  stream,                       /**< [in]     Input JPEG-HDR byte stream. */
    jh_monitor_t *monitor,                      /**< [in]     Progress status of the function. Initial value of the jh_monitor_ts
                                                              @c cancel field must be 0. */
    const jh_decoder_params_t *decoder_params,  /**< [in]     Decoder parameters. */
    jh_error_t* error                           /**< [in,out] User-allocated error-reporting structure that contains information
                                                              about the error. */
);

/** @brief Returns the decoder library version in a formatted string of @c major.minor.build.
 *  @return The library version in a null-terminated character string.
 */
const char* jh_decoder_version();

/** @brief Determines if @c stream contains a JPEG-HDR file.
 *  @attention jh_decode() should not be called if this function returns 0.
 *  @return 0 if no JPEG-HDR file is detected.
 */
int jh_is_hdr(
    jh_stream_t*  stream                       /**< [in] Input JPEG byte stream. */
);

/** @brief Determines the size of the image stored in the JPEG-HDR stream
 *  @return Description of the return value, which is a string.
 */
int jh_get_size(
    jh_stream_t*  stream,                       /**< [in]     Input JPEG byte stream. */
    int *columns,                               /**< [out]    Number of columns in the image. */
    int *rows,                                  /**< [out]    Number of rows in the image. */
    int *channels,                              /**< [out]    Number of channels in the image. */
    jh_error_t* error                           /**< [in,out] User-allocated error-reporting structure that contains information
                                                              about the error. */
);
    
/** @brief Sets the default value of the decoder parameters.
 *  @return 0 if the function does not succeed.
 */
int jh_decoder_set_default(
    jh_decoder_params_t *params,                /**< [out]    User-allocated decoder parameters.  */
    jh_error_t *error                           /**< [in,out] User-allocated error-reporting structure that contains information
                                                about the error. */
);


/**@}*/  /* end of defgroup decodint */

#ifdef __cplusplus
}
#endif

#endif /*_JH_DECODER_H*/
